// This file is a test of a 4 Digit LCD Display Using Driver

#include <mega128.h>   
#include <delay.h>

// device selection  

// lcd_d* is in BCD encoding

// D0  D1  D2  D3  Output
// ----------------------
// 0   0   0   0   0
// 0   0   0   1   1
// 0   0   1   0   2
// 0   0   1   1   3
// 0   1   0   0   4
// 0   1   0   1   5
// 0   1   1   0   6
// 0   1   1   1   7
// 1   0   0   0   8
// 1   0   0   1   9
// 1   0   1   0   -
// 1   0   1   1   E
// 1   1   0   0   H
// 1   1   0   1   L
// 1   1   1   0   P
// 1   1   1   1   BLANK   

#define BEGIN {
#define END }
 


// lcd_DIGIT1  lcd_DIGIT2    Selected
//	0	0              1000
//	0	1              0100
//	1	0              0010
//	1	1              0001


#define lcd_DIGIT1 PORTA.5   
#define lcd_DIGIT2 PORTA.4

#define lcd_CS1 PORTA.6
#define lcd_CS2 PORTA.7

void lcd_init(void);
void lcd_4digit_clear(void);
void lcd_4digit_dash(void);
void lcd_4digit_error(unsigned int lcd_value);
void lcd_4digit_marked(unsigned int lcd_value);
void lcd_4digit_int(unsigned int lcd_value);    
void lcd_4digit_help(void);         


char lower_lsb, upper_lsb, lower_msb, upper_msb;
int lcd_bcd;


void lcd_init(void)
BEGIN   
lcd_CS1=0;
lcd_CS2=0;   
DDRA = 0xFF;   
END





///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void lcd_4digit_int(unsigned int lcd_value)
BEGIN
                

// signed int16 lcd_value to int16 lcd_bcd (BCD) conversion
// signed int16 lcd_value;
// int16 lcd_bcd;
// MSB 0xffff LSB
// 0 to 15


lcd_bcd=0;


// Convert To BCD
while(lcd_value>=10000){lcd_value-=10000;}
while(lcd_value>=1000){lcd_value-=1000;lcd_bcd+=0x1000;}	
while(lcd_value>=100){lcd_value-=100;lcd_bcd+=0x0100;}
while(lcd_value>=10){lcd_value-=10;lcd_bcd+=0x0010;}
while(lcd_value>=1){lcd_value-=1;lcd_bcd+=0x0001;}



lower_lsb=lcd_bcd&0x000f;
lcd_bcd>>=4;
upper_lsb=lcd_bcd&0x000f;
lcd_bcd>>=4;
lower_msb=lcd_bcd&0x000f;
lcd_bcd>>=4;
upper_msb=lcd_bcd&0x000f;    




lcd_CS1=0;   
PORTA = lower_lsb;  
lcd_DIGIT1=0;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;        

lcd_CS1=0;   
PORTA = upper_lsb;  
lcd_DIGIT1=1;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;  

lcd_CS1=0;   
PORTA = lower_msb;  
lcd_DIGIT1=0;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    

lcd_CS1=0;   
PORTA = upper_msb;  
lcd_DIGIT1=1;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0; 


END           

void lcd_4digit_help(void)
BEGIN
lcd_CS1=0;   
PORTA = 14;  
lcd_DIGIT1=0;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;        

lcd_CS1=0;   
PORTA = 13;  
lcd_DIGIT1=1;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;  

lcd_CS1=0;   
PORTA = 11;  
lcd_DIGIT1=0;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    

lcd_CS1=0;   
PORTA = 12;  
lcd_DIGIT1=1;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    


END        

void lcd_4digit_clear(void)
BEGIN
lcd_CS1=0;   
PORTA = 15;  
lcd_DIGIT1=0;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;        

lcd_CS1=0;   
PORTA = 15;  
lcd_DIGIT1=1;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;  

lcd_CS1=0;   
PORTA = 15;  
lcd_DIGIT1=0;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    

lcd_CS1=0;   
PORTA = 15;  
lcd_DIGIT1=1;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    


END
          

void lcd_4digit_dash(void)
BEGIN
lcd_CS1=0;   
PORTA = 10;  
lcd_DIGIT1=0;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;        

lcd_CS1=0;   
PORTA = 10;  
lcd_DIGIT1=1;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;  

lcd_CS1=0;   
PORTA = 10;  
lcd_DIGIT1=0;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    

lcd_CS1=0;   
PORTA = 10;  
lcd_DIGIT1=1;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    


END

void lcd_4digit_error(unsigned int lcd_value)
BEGIN

// signed int16 lcd_value to int16 lcd_bcd (BCD) conversion
// signed int16 lcd_value;
// int16 lcd_bcd;
// MSB 0xffff LSB
// 0 to 15


lcd_bcd=0;


// Convert To BCD
while(lcd_value>=10000){lcd_value-=10000;}
while(lcd_value>=1000){lcd_value-=1000;}	
while(lcd_value>=100){lcd_value-=100;}
while(lcd_value>=10){lcd_value-=10;lcd_bcd+=0x0010;}
while(lcd_value>=1){lcd_value-=1;lcd_bcd+=0x0001;}



lower_lsb=lcd_bcd&0x000f;
lcd_bcd>>=4;
upper_lsb=lcd_bcd&0x000f;
lcd_bcd>>=4;
lower_msb=lcd_bcd&0x000f;
lcd_bcd>>=4;
upper_msb=lcd_bcd&0x000f;    




lcd_CS1=0;   
PORTA = lower_lsb;  
lcd_DIGIT1=0;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;        

lcd_CS1=0;   
PORTA = upper_lsb;  
lcd_DIGIT1=1;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    


lcd_CS1=0;   
PORTA = lower_msb;  
lcd_DIGIT1=0;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;  

/*lcd_CS1=0;   
PORTA = 15;  
lcd_DIGIT1=0;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    
*/
lcd_CS1=0;   
PORTA = 11;  
lcd_DIGIT1=1;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0; 


END                 


void lcd_4digit_marked(unsigned int lcd_value)
BEGIN

// signed int16 lcd_value to int16 lcd_bcd (BCD) conversion
// signed int16 lcd_value;
// int16 lcd_bcd;
// MSB 0xffff LSB
// 0 to 15


lcd_bcd=0;


// Convert To BCD
while(lcd_value>=10000){lcd_value-=10000;}
while(lcd_value>=1000){lcd_value-=1000;}	
while(lcd_value>=100){lcd_value-=100;}
while(lcd_value>=10){lcd_value-=10;lcd_bcd+=0x0010;}
while(lcd_value>=1){lcd_value-=1;lcd_bcd+=0x0001;}



lower_lsb=lcd_bcd&0x000f;
lcd_bcd>>=4;
upper_lsb=lcd_bcd&0x000f;
lcd_bcd>>=4;
lower_msb=lcd_bcd&0x000f;
lcd_bcd>>=4;
upper_msb=lcd_bcd&0x000f;    




lcd_CS1=0;   
PORTA = 10;  
lcd_DIGIT1=0;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;        

lcd_CS1=0;   
PORTA = lower_lsb;  
lcd_DIGIT1=1;
lcd_DIGIT2=0;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    



lcd_CS1=0;   
PORTA =  upper_lsb;  
lcd_DIGIT1=0;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0;    

lcd_CS1=0;   
PORTA = 10;  
lcd_DIGIT1=1;
lcd_DIGIT2=1;
lcd_CS1=1;  
delay_ms(5);
lcd_CS1=0; 


END           